@@ -1,14 +1,14 @@
 
 // This is from https://code.google.com/p/mman-win32/
-// 
+//
 // Licensed under MIT
 
 #ifndef _MMAN_WIN32_H
 #define _MMAN_WIN32_H
 
-#ifndef _WIN32_WINNT		// Allow use of features specific to Windows XP or later.                   
+#ifndef _WIN32_WINNT		// Allow use of features specific to Windows XP or later.
 #define _WIN32_WINNT 0x0501	// Change this to the appropriate value to target other versions of Windows.
-#endif						
+#endif
 
 #include <sys/types.h>
 #include <windows.h>
@@ -50,13 +50,13 @@ static int __map_mman_error(const DWORD err, const int deferr)
 static DWORD __map_mmap_prot_page(const int prot)
 {
     DWORD protect = 0;
-    
+
     if (prot == PROT_NONE)
         return protect;
-        
+
     if ((prot & PROT_EXEC) != 0)
     {
-        protect = ((prot & PROT_WRITE) != 0) ? 
+        protect = ((prot & PROT_WRITE) != 0) ?
                     PAGE_EXECUTE_READWRITE : PAGE_EXECUTE_READ;
     }
     else
@@ -64,39 +64,39 @@ static DWORD __map_mmap_prot_page(const int prot)
         protect = ((prot & PROT_WRITE) != 0) ?
                     PAGE_READWRITE : PAGE_READONLY;
     }
-    
+
     return protect;
 }
 
 static DWORD __map_mmap_prot_file(const int prot)
 {
     DWORD desiredAccess = 0;
-    
+
     if (prot == PROT_NONE)
         return desiredAccess;
-        
+
     if ((prot & PROT_READ) != 0)
         desiredAccess |= FILE_MAP_READ;
     if ((prot & PROT_WRITE) != 0)
         desiredAccess |= FILE_MAP_WRITE;
     if ((prot & PROT_EXEC) != 0)
         desiredAccess |= FILE_MAP_EXECUTE;
-    
+
     return desiredAccess;
 }
 
 inline void* mmap(void *addr, size_t len, int prot, int flags, int fildes, off_t off)
 {
     HANDLE fm, h;
-    
+
     void * map = MAP_FAILED;
-    
+
 #ifdef _MSC_VER
 #pragma warning(push)
 #pragma warning(disable: 4293)
 #endif
 
-    const DWORD dwFileOffsetLow = (sizeof(off_t) <= sizeof(DWORD)) ? 
+    const DWORD dwFileOffsetLow = (sizeof(off_t) <= sizeof(DWORD)) ?
                     (DWORD)off : (DWORD)(off & 0xFFFFFFFFL);
     const DWORD dwFileOffsetHigh = (sizeof(off_t) <= sizeof(DWORD)) ?
                     (DWORD)0 : (DWORD)((off >> 32) & 0xFFFFFFFFL);
@@ -105,7 +105,7 @@ inline void* mmap(void *addr, size_t len, int prot, int flags, int fildes, off_t
 
     const off_t maxSize = off + (off_t)len;
 
-    const DWORD dwMaxSizeLow = (sizeof(off_t) <= sizeof(DWORD)) ? 
+    const DWORD dwMaxSizeLow = (sizeof(off_t) <= sizeof(DWORD)) ?
                     (DWORD)maxSize : (DWORD)(maxSize & 0xFFFFFFFFL);
     const DWORD dwMaxSizeHigh = (sizeof(off_t) <= sizeof(DWORD)) ?
                     (DWORD)0 : (DWORD)((maxSize >> 32) & 0xFFFFFFFFL);
@@ -115,8 +115,8 @@ inline void* mmap(void *addr, size_t len, int prot, int flags, int fildes, off_t
 #endif
 
     errno = 0;
-    
-    if (len == 0 
+
+    if (len == 0
         /* Unsupported flag combinations */
         || (flags & MAP_FIXED) != 0
         /* Usupported protection combinations */
@@ -125,8 +125,8 @@ inline void* mmap(void *addr, size_t len, int prot, int flags, int fildes, off_t
         errno = EINVAL;
         return MAP_FAILED;
     }
-    
-    h = ((flags & MAP_ANONYMOUS) == 0) ? 
+
+    h = ((flags & MAP_ANONYMOUS) == 0) ?
                     (HANDLE)_get_osfhandle(fildes) : INVALID_HANDLE_VALUE;
 
     if ((flags & MAP_ANONYMOUS) == 0 && h == INVALID_HANDLE_VALUE)
@@ -142,11 +142,11 @@ inline void* mmap(void *addr, size_t len, int prot, int flags, int fildes, off_t
         errno = __map_mman_error(GetLastError(), EPERM);
         return MAP_FAILED;
     }
-  
+
     map = MapViewOfFile(fm, desiredAccess, dwFileOffsetHigh, dwFileOffsetLow, len);
 
     CloseHandle(fm);
-  
+
     if (map == NULL)
     {
         errno = __map_mman_error(GetLastError(), EPERM);
@@ -160,9 +160,9 @@ inline int munmap(void *addr, size_t len)
 {
     if (UnmapViewOfFile(addr))
         return 0;
-        
+
     errno =  __map_mman_error(GetLastError(), EPERM);
-    
+
     return -1;
 }
 
@@ -170,12 +170,12 @@ inline int mprotect(void *addr, size_t len, int prot)
 {
     DWORD newProtect = __map_mmap_prot_page(prot);
     DWORD oldProtect = 0;
-    
+
     if (VirtualProtect(addr, len, newProtect, &oldProtect))
         return 0;
-    
+
     errno =  __map_mman_error(GetLastError(), EPERM);
-    
+
     return -1;
 }
 
@@ -183,9 +183,9 @@ inline int msync(void *addr, size_t len, int flags)
 {
     if (FlushViewOfFile(addr, len))
         return 0;
-    
+
     errno =  __map_mman_error(GetLastError(), EPERM);
-    
+
     return -1;
 }
 
@@ -193,9 +193,9 @@ inline int mlock(const void *addr, size_t len)
 {
     if (VirtualLock((LPVOID)addr, len))
         return 0;
-        
+
     errno =  __map_mman_error(GetLastError(), EPERM);
-    
+
     return -1;
 }
 
@@ -203,10 +203,36 @@ inline int munlock(const void *addr, size_t len)
 {
     if (VirtualUnlock((LPVOID)addr, len))
         return 0;
-        
+
     errno =  __map_mman_error(GetLastError(), EPERM);
-    
+
     return -1;
 }
 
-#endif 
+#if !defined(__MINGW32__)
+int ftruncate(int fd, unsigned int size) {
+    if (fd < 0) {
+        errno = EBADF;
+        return -1;
+    }
+
+    HANDLE h = (HANDLE)_get_osfhandle(fd);
+    unsigned int cur = SetFilePointer(h, 0, NULL, FILE_CURRENT);
+    if (cur == ~0 || SetFilePointer(h, size, NULL, FILE_BEGIN) == ~0 || !SetEndOfFile(h)) {
+        int error = GetLastError();
+        switch (GetLastError()) {
+            case ERROR_INVALID_HANDLE:
+                errno = EBADF;
+                break;
+            default:
+                errno = EIO;
+                break;
+        }
+        return -1;
+    }
+
+    return 0;
+}
+#endif
+
+#endif
